==> utiles-0.3.0.tar.gz: utiles-0.3.0/utiles-pyo3/src/pyutiles/pylnglatbbox.rs <==
use crate::pyutiles::pyiters::FloatIterator;
use crate::pyutiles::pytile::PyTile;
use pyo3::basic::CompareOp;
use pyo3::types::PyType;
use pyo3::{
    exceptions, pyclass, pymethods, IntoPy, Py, PyAny, PyErr, PyObject, PyRef,
    PyResult, Python,
};
use utiles;
use utiles::bbox::BBox;

#[pyclass(name = "LngLatBbox")]
#[derive(Clone)]
pub struct PyLngLatBbox {
    pub bbox: BBox,
}

#[pymethods]
impl PyLngLatBbox {
    #[new]
    pub fn new(west: f64, south: f64, east: f64, north: f64) -> Self {
        PyLngLatBbox {
            bbox: BBox {
                north,
                south,
                east,
                west,
            },
        }
    }

    pub fn __iter__(slf: PyRef<'_, Self>) -> PyResult<Py<FloatIterator>> {
        let iter = FloatIterator {
            iter: Box::new(
                vec![
                    slf.bbox.west(),
                    slf.bbox.south(),
                    slf.bbox.east(),
                    slf.bbox.north(),
                ]
                .into_iter(),
            ),
        };
        Py::new(slf.py(), iter)
    }

    pub fn __repr__(&self) -> String {
        format!(
            "LngLatBbox(west={}, south={}, east={}, north={})",
            self.bbox.west(),
            self.bbox.south(),
            self.bbox.east(),
            self.bbox.north()
        )
    }

    pub fn __str__(&self) -> String {
        self.__repr__()
    }

    #[classmethod]
    pub fn from_tile(_cls: &PyType, tile: &PyTile) -> Self {
        let ul = utiles::ul(tile.xyz.x, tile.xyz.y, tile.xyz.z);
        let lr = utiles::lr(tile.xyz.x, tile.xyz.y, tile.xyz.z);
        Self::new(ul.lng(), lr.lat(), lr.lng(), ul.lat())
    }

    #[getter]
    pub fn west(&self) -> f64 {
        self.bbox.west()
    }

    #[getter]
    pub fn south(&self) -> f64 {
        self.bbox.south()
    }

    #[getter]
    pub fn east(&self) -> f64 {
        self.bbox.east()
    }

    #[getter]
    pub fn north(&self) -> f64 {
        self.bbox.north()
    }

    pub fn members(&self) -> (f64, f64, f64, f64) {
        self.tuple()
    }

    pub fn __len__(&self) -> usize {
        4
    }

    pub fn __getitem__(&self, idx: i32, _py: Python<'_>) -> PyResult<f64> {
        match idx {
            0 => Ok(self.bbox.west),
            1 => Ok(self.bbox.south),
            2 => Ok(self.bbox.east),
            3 => Ok(self.bbox.north),
            -1 => Ok(self.bbox.north),
            -2 => Ok(self.bbox.east),
            -3 => Ok(self.bbox.south),
            -4 => Ok(self.bbox.west),
            4 => Err(PyErr::new::<exceptions::PyStopIteration, _>("")),

            _ => panic!("Index {idx} out of range for tile"),
        }
    }

    pub fn tuple(&self) -> (f64, f64, f64, f64) {
        self.bbox.tuple()
    }

    pub fn __richcmp__(
        &self,
        other: &PyAny,
        op: CompareOp,
        py: Python<'_>,
    ) -> PyObject {
        let maybetuple = other.extract::<(f64, f64, f64, f64)>();

        if let Ok(tuple) = maybetuple {
            match op {
                CompareOp::Eq => (self.bbox.west() == tuple.0
                    && self.bbox.south() == tuple.1
                    && self.bbox.east() == tuple.2
                    && self.bbox.north() == tuple.3)
                    .into_py(py),
                CompareOp::Ne => (self.bbox.west() != tuple.0
                    || self.bbox.south() != tuple.1
                    || self.bbox.east() != tuple.2
                    || self.bbox.north() != tuple.3)
                    .into_py(py),
                CompareOp::Lt => (self.bbox.west() < tuple.0
                    || self.bbox.south() < tuple.1
                    || self.bbox.east() < tuple.2
                    || self.bbox.north() < tuple.3)
                    .into_py(py),
                _ => py.NotImplemented(),
            }
        } else {
            let other = other.extract::<PyRef<PyLngLatBbox>>().unwrap();
            match op {
                CompareOp::Eq => (self.bbox.west() == other.bbox.west()
                    && self.bbox.south() == other.bbox.south()
                    && self.bbox.east() == other.bbox.east()
                    && self.bbox.north() == other.bbox.north())
                .into_py(py),
                CompareOp::Ne => (self.bbox.west != other.bbox.west()
                    || self.bbox.south() != other.bbox.south()
                    || self.bbox.east() != other.bbox.east()
                    || self.bbox.north() != other.bbox.north())
                .into_py(py),
                CompareOp::Lt => (self.bbox.west() < other.bbox.west()
                    || self.bbox.south() < other.bbox.south()
                    || self.bbox.east() < other.bbox.east()
                    || self.bbox.north() < other.bbox.north())
                .into_py(py),
                _ => py.NotImplemented(),
            }
        }
    }
}

==> utiles-0.4.0.tar.gz: utiles-0.4.0/utiles-pyo3/src/pyutiles/pylnglatbbox.rs <==
use crate::pyutiles::pyiters::FloatIterator;
use crate::pyutiles::pytile::PyTile;

use pyo3::basic::CompareOp;
use pyo3::prelude::*;
use pyo3::types::PyType;
use pyo3::{
    exceptions, pyclass, pymethods, IntoPy, Py, PyAny, PyErr, PyObject, PyRef,
    PyResult, Python,
};
use utiles;
use utiles::bbox::BBox;

#[pyclass(name = "LngLatBbox")]
#[derive(Clone)]
pub struct PyLngLatBbox {
    pub bbox: BBox,
}

#[pymethods]
impl PyLngLatBbox {
    #[new]
    pub fn new(west: f64, south: f64, east: f64, north: f64) -> Self {
        PyLngLatBbox {
            bbox: BBox {
                north,
                south,
                east,
                west,
            },
        }
    }

    pub fn __iter__(slf: PyRef<'_, Self>) -> PyResult<Py<FloatIterator>> {
        let iter = FloatIterator {
            iter: Box::new(
                vec![
                    slf.bbox.west(),
                    slf.bbox.south(),
                    slf.bbox.east(),
                    slf.bbox.north(),
                ]
                .into_iter(),
            ),
        };
        Py::new(slf.py(), iter)
    }

    pub fn __repr__(&self) -> String {
        format!(
            "LngLatBbox(west={}, south={}, east={}, north={})",
            self.bbox.west(),
            self.bbox.south(),
            self.bbox.east(),
            self.bbox.north()
        )
    }

    pub fn __str__(&self) -> String {
        self.__repr__()
    }

    #[classmethod]
    pub fn from_tile(_cls: &Bound<'_, PyType>, tile: &PyTile) -> Self {
        let ul = utiles::ul(tile.xyz.x, tile.xyz.y, tile.xyz.z);
        let lr = utiles::lr(tile.xyz.x, tile.xyz.y, tile.xyz.z);
        Self::new(ul.lng(), lr.lat(), lr.lng(), ul.lat())
    }

    #[getter]
    pub fn west(&self) -> f64 {
        self.bbox.west()
    }

    #[getter]
    pub fn south(&self) -> f64 {
        self.bbox.south()
    }

    #[getter]
    pub fn east(&self) -> f64 {
        self.bbox.east()
    }

    #[getter]
    pub fn north(&self) -> f64 {
        self.bbox.north()
    }

    pub fn members(&self) -> (f64, f64, f64, f64) {
        self.tuple()
    }

    pub fn __len__(&self) -> usize {
        4
    }

    pub fn __getitem__(&self, idx: i32, _py: Python<'_>) -> PyResult<f64> {
        match idx {
            0 => Ok(self.bbox.west),
            1 => Ok(self.bbox.south),
            2 => Ok(self.bbox.east),
            3 => Ok(self.bbox.north),
            -1 => Ok(self.bbox.north),
            -2 => Ok(self.bbox.east),
            -3 => Ok(self.bbox.south),
            -4 => Ok(self.bbox.west),
            4 => Err(PyErr::new::<exceptions::PyStopIteration, _>("")),

            _ => panic!("Index {idx} out of range for tile"),
        }
    }

    pub fn tuple(&self) -> (f64, f64, f64, f64) {
        self.bbox.tuple()
    }

    pub fn __richcmp__(
        &self,
        other: &Bound<'_, PyAny>,
        op: CompareOp,
        py: Python<'_>,
    ) -> PyObject {
        let maybetuple = other.extract::<(f64, f64, f64, f64)>();

        if let Ok(tuple) = maybetuple {
            match op {
                CompareOp::Eq => (self.bbox.west() == tuple.0
                    && self.bbox.south() == tuple.1
                    && self.bbox.east() == tuple.2
                    && self.bbox.north() == tuple.3)
                    .into_py(py),
                CompareOp::Ne => (self.bbox.west() != tuple.0
                    || self.bbox.south() != tuple.1
                    || self.bbox.east() != tuple.2
                    || self.bbox.north() != tuple.3)
                    .into_py(py),
                CompareOp::Lt => (self.bbox.west() < tuple.0
                    || self.bbox.south() < tuple.1
                    || self.bbox.east() < tuple.2
                    || self.bbox.north() < tuple.3)
                    .into_py(py),
                _ => py.NotImplemented(),
            }
        } else {
            let other = other.extract::<PyRef<PyLngLatBbox>>().unwrap();
            match op {
                CompareOp::Eq => (self.bbox.west() == other.bbox.west()
                    && self.bbox.south() == other.bbox.south()
                    && self.bbox.east() == other.bbox.east()
                    && self.bbox.north() == other.bbox.north())
                .into_py(py),
                CompareOp::Ne => (self.bbox.west != other.bbox.west()
                    || self.bbox.south() != other.bbox.south()
                    || self.bbox.east() != other.bbox.east()
                    || self.bbox.north() != other.bbox.north())
                .into_py(py),
                CompareOp::Lt => (self.bbox.west() < other.bbox.west()
                    || self.bbox.south() < other.bbox.south()
                    || self.bbox.east() < other.bbox.east()
                    || self.bbox.north() < other.bbox.north())
                .into_py(py),
                _ => py.NotImplemented(),
            }
        }
    }
}
